@@ -3,7 +3,8 @@
 /*
  * This file is part of the Predis package.
  *
- * (c) Daniele Alessandri <suppakilla@gmail.com>
+ * (c) 2009-2020 Daniele Alessandri
+ * (c) 2021-2023 Till Krüss
  *
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
@@ -12,16 +13,15 @@
 namespace Predis\Connection;
 
 use PredisTestCase;
+use ReflectionObject;
+use stdClass;
 
-/**
- *
- */
 class FactoryTest extends PredisTestCase
 {
     /**
      * @group disconnected
      */
-    public function testImplementsCorrectInterface()
+    public function testImplementsCorrectInterface(): void
     {
         $factory = new Factory();
 
@@ -31,35 +31,35 @@ class FactoryTest extends PredisTestCase
     /**
      * @group disconnected
      */
-    public function testSettingDefaultParameters()
+    public function testSettingDefaultParameters(): void
     {
         $factory = new Factory();
 
-        $factory->setDefaultParameters($defaults = array(
+        $factory->setDefaultParameters($defaults = [
             'password' => 'secret',
             'database' => 10,
             'custom' => 'foobar',
-        ));
+        ]);
 
         $this->assertSame($defaults, $factory->getDefaultParameters());
 
-        $parameters = array('database' => 10, 'persistent' => true);
+        $parameters = ['database' => 10, 'persistent' => true];
     }
 
     /**
      * @group disconnected
      */
-    public function testCreateTcpConnection()
+    public function testCreateTcpConnection(): void
     {
         $factory = new Factory();
 
-        $parameters = new Parameters(array('scheme' => 'tcp'));
+        $parameters = new Parameters(['scheme' => 'tcp']);
         $connection = $factory->create($parameters);
 
         $this->assertInstanceOf('Predis\Connection\StreamConnection', $connection);
         $this->assertSame($parameters, $connection->getParameters());
 
-        $parameters = new Parameters(array('scheme' => 'redis'));
+        $parameters = new Parameters(['scheme' => 'redis']);
         $connection = $factory->create($parameters);
 
         $this->assertInstanceOf('Predis\Connection\StreamConnection', $connection);
@@ -69,17 +69,17 @@ class FactoryTest extends PredisTestCase
     /**
      * @group disconnected
      */
-    public function testCreateSslConnection()
+    public function testCreateSslConnection(): void
     {
         $factory = new Factory();
 
-        $parameters = new Parameters(array('scheme' => 'tls'));
+        $parameters = new Parameters(['scheme' => 'tls']);
         $connection = $factory->create($parameters);
 
         $this->assertInstanceOf('Predis\Connection\StreamConnection', $connection);
         $this->assertSame($parameters, $connection->getParameters());
 
-        $parameters = new Parameters(array('scheme' => 'rediss'));
+        $parameters = new Parameters(['scheme' => 'rediss']);
         $connection = $factory->create($parameters);
 
         $this->assertInstanceOf('Predis\Connection\StreamConnection', $connection);
@@ -89,11 +89,11 @@ class FactoryTest extends PredisTestCase
     /**
      * @group disconnected
      */
-    public function testCreateUnixConnection()
+    public function testCreateUnixConnection(): void
     {
         $factory = new Factory();
 
-        $parameters = new Parameters(array('scheme' => 'unix', 'path' => '/tmp/redis.sock'));
+        $parameters = new Parameters(['scheme' => 'unix', 'path' => '/tmp/redis.sock']);
         $connection = $factory->create($parameters);
 
         $this->assertInstanceOf('Predis\Connection\StreamConnection', $connection);
@@ -103,21 +103,21 @@ class FactoryTest extends PredisTestCase
     /**
      * @group disconnected
      */
-    public function testCreateConnectionWithParametersInstanceAndDefaultsDoesNotAlterOriginalParameters()
+    public function testCreateConnectionWithParametersInstanceAndDefaultsDoesNotAlterOriginalParameters(): void
     {
         $factory = new Factory();
 
-        $factory->setDefaultParameters($defaultParams = array(
+        $factory->setDefaultParameters($defaultParams = [
             'port' => 7000,
             'password' => 'secret',
             'database' => 10,
             'custom' => 'foobar',
-        ));
+        ]);
 
-        $inputParams = new Parameters(array(
+        $inputParams = new Parameters([
             'host' => 'localhost',
             'database' => 5,
-        ));
+        ]);
 
         $connection = $factory->create($inputParams);
         $parameters = $connection->getParameters();
@@ -136,7 +136,7 @@ class FactoryTest extends PredisTestCase
     /**
      * @group disconnected
      */
-    public function testCreateConnectionWithNullParameters()
+    public function testCreateConnectionWithNullParameters(): void
     {
         $factory = new Factory();
         $connection = $factory->create(null);
@@ -152,15 +152,15 @@ class FactoryTest extends PredisTestCase
     /**
      * @group disconnected
      */
-    public function testCreateConnectionWithNullParametersAndDefaults()
+    public function testCreateConnectionWithNullParametersAndDefaults(): void
     {
         $factory = new Factory();
 
-        $factory->setDefaultParameters($defaultParams = array(
+        $factory->setDefaultParameters($defaultParams = [
             'port' => 7000,
             'password' => 'secret',
             'custom' => 'foobar',
-        ));
+        ]);
 
         $connection = $factory->create(null);
         $parameters = $connection->getParameters();
@@ -177,10 +177,10 @@ class FactoryTest extends PredisTestCase
     /**
      * @group disconnected
      */
-    public function testCreateConnectionWithArrayParameters()
+    public function testCreateConnectionWithArrayParameters(): void
     {
         $factory = new Factory();
-        $connection = $factory->create(array('scheme' => 'tcp', 'custom' => 'foobar'));
+        $connection = $factory->create(['scheme' => 'tcp', 'custom' => 'foobar']);
         $parameters = $connection->getParameters();
 
         $this->assertInstanceOf('Predis\Connection\NodeConnectionInterface', $connection);
@@ -193,21 +193,21 @@ class FactoryTest extends PredisTestCase
     /**
      * @group disconnected
      */
-    public function testCreateConnectionWithArrayParametersAndDefaults()
+    public function testCreateConnectionWithArrayParametersAndDefaults(): void
     {
         $factory = new Factory();
 
-        $factory->setDefaultParameters($defaultParams = array(
+        $factory->setDefaultParameters($defaultParams = [
             'port' => 7000,
             'password' => 'secret',
             'custom' => 'foobar',
-        ));
+        ]);
 
-        $connection = $factory->create($inputParams = array(
+        $connection = $factory->create($inputParams = [
             'host' => 'localhost',
             'port' => 8000,
             'persistent' => true,
-        ));
+        ]);
 
         $parameters = $connection->getParameters();
 
@@ -224,7 +224,7 @@ class FactoryTest extends PredisTestCase
     /**
      * @group disconnected
      */
-    public function testCreateConnectionWithStringURI()
+    public function testCreateConnectionWithStringURI(): void
     {
         $factory = new Factory();
         $connection = $factory->create('tcp://127.0.0.1?custom=foobar');
@@ -240,15 +240,15 @@ class FactoryTest extends PredisTestCase
     /**
      * @group disconnected
      */
-    public function testCreateConnectionWithStrinURIAndDefaults()
+    public function testCreateConnectionWithStrinURIAndDefaults(): void
     {
         $factory = new Factory();
 
-        $factory->setDefaultParameters($defaultParams = array(
+        $factory->setDefaultParameters($defaultParams = [
             'port' => 7000,
             'password' => 'secret',
             'custom' => 'foobar',
-        ));
+        ]);
 
         $connection = $factory->create('tcp://localhost:8000?persistent=1');
         $parameters = $connection->getParameters();
@@ -266,43 +266,53 @@ class FactoryTest extends PredisTestCase
     /**
      * @group disconnected
      */
-    public function testCreateConnectionWithoutInitializationCommands()
+    public function testCreateConnectionWithoutInitializationCommands(): void
     {
-        $profile = $this->getMock('Predis\Profile\ProfileInterface');
-        $profile->expects($this->never())->method('createCommand');
+        $connection = $this->getMockBuilder('Predis\Connection\NodeConnectionInterface')->getMock();
+        $connection
+            ->expects($this->never())
+            ->method('addConnectCommand');
 
-        $factory = new Factory($profile);
-        $parameters = new Parameters();
-        $connection = $factory->create($parameters);
+        $parameters = new Parameters(['scheme' => 'test']);
 
-        $this->assertInstanceOf('Predis\Connection\NodeConnectionInterface', $connection);
+        $factory = new Factory();
+        $factory->define('test', function ($_parameters, $_factory) use ($connection, $parameters, $factory) {
+            $this->assertSame($_parameters, $parameters);
+            $this->assertSame($_factory, $factory);
+
+            return $connection;
+        });
+
+        $this->assertSame($connection, $factory->create($parameters));
     }
 
     /**
      * @group disconnected
      */
-    public function testCreateConnectionWithInitializationCommands()
+    public function testCreateConnectionWithInitializationCommands(): void
     {
-        $parameters = new Parameters(array(
+        $parameters = new Parameters([
             'database' => '0',
             'password' => 'foobar',
-        ));
+        ]);
 
-        $connection = $this->getMock('Predis\Connection\NodeConnectionInterface');
-        $connection->expects($this->once())
-                   ->method('getParameters')
-                   ->will($this->returnValue($parameters));
-        $connection->expects($this->at(1))
-                   ->method('addConnectCommand')
-                   ->with($this->isRedisCommand('AUTH', array('foobar')));
-        $connection->expects($this->at(2))
-                   ->method('addConnectCommand')
-                   ->with($this->isRedisCommand('SELECT', array(0)));
+        $connection = $this->getMockBuilder('Predis\Connection\NodeConnectionInterface')->getMock();
+        $connection
+            ->expects($this->once())
+            ->method('getParameters')
+            ->willReturn($parameters);
+        $connection
+            ->expects($this->exactly(2))
+            ->method('addConnectCommand')
+            ->withConsecutive(
+                [$this->isRedisCommand('AUTH', ['foobar'])],
+                [$this->isRedisCommand('SELECT', ['0'])]
+            );
 
         $factory = new Factory();
 
         // TODO: using reflection to make a protected method accessible :facepalm:
-        $reflection = new \ReflectionObject($factory);
+        $reflection = new ReflectionObject($factory);
         $prepareConnection = $reflection->getMethod('prepareConnection');
         $prepareConnection->setAccessible(true);
         $prepareConnection->invoke($factory, $connection);
@@ -313,22 +323,22 @@ class FactoryTest extends PredisTestCase
      */
     public function testCreateConnectionWithPasswordAndNoUsernameAddsInitializationCommandAuthWithOneArgument()
     {
-        $parameters = new Parameters(array(
+        $parameters = new Parameters([
             'password' => 'foobar',
-        ));
+        ]);
 
-        $connection = $this->getMock('Predis\Connection\NodeConnectionInterface');
+        $connection = $this->getMockBuilder('Predis\Connection\NodeConnectionInterface')->getMock();
         $connection->expects($this->once())
             ->method('getParameters')
             ->will($this->returnValue($parameters));
-        $connection->expects($this->once(1))
+        $connection->expects($this->once())
             ->method('addConnectCommand')
-            ->with($this->isRedisCommand('AUTH', array('foobar')));
+            ->with($this->isRedisCommand('AUTH', ['foobar']));
 
         $factory = new Factory();
 
         // TODO: using reflection to make a protected method accessible :facepalm:
-        $reflection = new \ReflectionObject($factory);
+        $reflection = new ReflectionObject($factory);
         $prepareConnection = $reflection->getMethod('prepareConnection');
         $prepareConnection->setAccessible(true);
         $prepareConnection->invoke($factory, $connection);
@@ -339,23 +349,23 @@ class FactoryTest extends PredisTestCase
      */
     public function testCreateConnectionWithPasswordAndUsernameAddsInitializationCommandAuthWithTwoArguments()
     {
-        $parameters = new Parameters(array(
+        $parameters = new Parameters([
             'username' => 'myusername',
             'password' => 'foobar',
-        ));
+        ]);
 
-        $connection = $this->getMock('Predis\Connection\NodeConnectionInterface');
+        $connection = $this->getMockBuilder('Predis\Connection\NodeConnectionInterface')->getMock();
         $connection->expects($this->once())
             ->method('getParameters')
             ->will($this->returnValue($parameters));
-        $connection->expects($this->once(1))
+        $connection->expects($this->once())
             ->method('addConnectCommand')
-            ->with($this->isRedisCommand('AUTH', array('myusername', 'foobar')));
+            ->with($this->isRedisCommand('AUTH', ['myusername', 'foobar']));
 
         $factory = new Factory();
 
         // TODO: using reflection to make a protected method accessible :facepalm:
-        $reflection = new \ReflectionObject($factory);
+        $reflection = new ReflectionObject($factory);
         $prepareConnection = $reflection->getMethod('prepareConnection');
         $prepareConnection->setAccessible(true);
         $prepareConnection->invoke($factory, $connection);
@@ -366,11 +376,11 @@ class FactoryTest extends PredisTestCase
      */
     public function testCreateConnectionWithUsernameAndNoPasswordDoesNotAddInitializationCommands()
     {
-        $parameters = new Parameters(array(
+        $parameters = new Parameters([
             'username' => 'myusername',
-        ));
+        ]);
 
-        $connection = $this->getMock('Predis\Connection\NodeConnectionInterface');
+        $connection = $this->getMockBuilder('Predis\Connection\NodeConnectionInterface')->getMock();
         $connection->expects($this->once())
             ->method('getParameters')
             ->will($this->returnValue($parameters));
@@ -380,7 +390,7 @@ class FactoryTest extends PredisTestCase
         $factory = new Factory();
 
         // TODO: using reflection to make a protected method accessible :facepalm:
-        $reflection = new \ReflectionObject($factory);
+        $reflection = new ReflectionObject($factory);
         $prepareConnection = $reflection->getMethod('prepareConnection');
         $prepareConnection->setAccessible(true);
         $prepareConnection->invoke($factory, $connection);
@@ -392,11 +402,11 @@ class FactoryTest extends PredisTestCase
      */
     public function testCreateConnectionWithEmptyParametersDoesNotAddInitializationCommands($parameter, $value)
     {
-        $parameters = new Parameters(array(
+        $parameters = new Parameters([
             $parameter => $value,
-        ));
+        ]);
 
-        $connection = $this->getMock('Predis\Connection\NodeConnectionInterface');
+        $connection = $this->getMockBuilder('Predis\Connection\NodeConnectionInterface')->getMock();
         $connection->expects($this->once())
             ->method('getParameters')
             ->will($this->returnValue($parameters));
@@ -406,7 +416,7 @@ class FactoryTest extends PredisTestCase
         $factory = new Factory();
 
         // TODO: using reflection to make a protected method accessible :facepalm:
-        $reflection = new \ReflectionObject($factory);
+        $reflection = new ReflectionObject($factory);
         $prepareConnection = $reflection->getMethod('prepareConnection');
         $prepareConnection->setAccessible(true);
         $prepareConnection->invoke($factory, $connection);
@@ -414,23 +424,24 @@ class FactoryTest extends PredisTestCase
 
     /**
      * @group disconnected
-     * @expectedException \InvalidArgumentException
-     * @expecteExceptionMessage Unknown connection scheme: 'unknown'.
      */
-    public function testCreateUndefinedConnection()
+    public function testCreateUndefinedConnection(): void
     {
+        $this->expectException('InvalidArgumentException');
+        $this->expectExceptionMessage("Unknown connection scheme: 'unknown'");
+
         $factory = new Factory();
-        $factory->create(new Parameters(array('scheme' => 'unknown')));
+        $factory->create(new Parameters(['scheme' => 'unknown']));
     }
 
     /**
      * @group disconnected
      */
-    public function testDefineConnectionWithFQN()
+    public function testDefineConnectionWithFQN(): void
     {
-        list(, $connectionClass) = $this->getMockConnectionClass();
+        [, $connectionClass] = $this->getMockConnectionClass();
 
-        $parameters = new Parameters(array('scheme' => 'foobar'));
+        $parameters = new Parameters(['scheme' => 'foobar']);
         $factory = new Factory();
 
         $factory->define($parameters->scheme, $connectionClass);
@@ -442,24 +453,28 @@ class FactoryTest extends PredisTestCase
     /**
      * @group disconnected
      */
-    public function testDefineConnectionWithCallable()
+    public function testDefineConnectionWithCallable(): void
     {
-        list(, $connectionClass) = $this->getMockConnectionClass();
+        [, $connectionClass] = $this->getMockConnectionClass();
 
-        $parameters = new Parameters(array('scheme' => 'foobar'));
+        $parameters = new Parameters(['scheme' => 'foobar']);
+        $factory = new Factory();
 
         $initializer = function ($parameters) use ($connectionClass) {
             return new $connectionClass($parameters);
         };
 
-        $initializerMock = $this->getMock('stdClass', array('__invoke'));
-        $initializerMock->expects($this->exactly(2))
-                        ->method('__invoke')
-                        ->with($parameters)
-                        ->will($this->returnCallback($initializer));
+        $initializerMock = $this->getMockBuilder('stdClass')
+            ->addMethods(['__invoke'])
+            ->getMock();
+        $initializerMock
+            ->expects($this->exactly(2))
+            ->method('__invoke')
+            ->with($parameters, $factory)
+            ->willReturnCallback($initializer);
 
-        $factory = new Factory();
         $factory->define($parameters->scheme, $initializerMock);
+
         $connection1 = $factory->create($parameters);
         $connection2 = $factory->create($parameters);
 
@@ -470,21 +485,23 @@ class FactoryTest extends PredisTestCase
 
     /**
      * @group disconnected
-     * @expectedException \InvalidArgumentException
      */
-    public function testDefineConnectionWithInvalidArgument()
+    public function testDefineConnectionWithInvalidArgument(): void
     {
+        $this->expectException('InvalidArgumentException');
+
         $factory = new Factory();
-        $factory->define('foobar', new \stdClass());
+        $factory->define('foobar', new stdClass());
     }
 
     /**
      * @group disconnected
-     * @expectedException \InvalidArgumentException
-     * @expecteExceptionMessage Unknown connection scheme: 'tcp'.
      */
-    public function testUndefineDefinedConnection()
+    public function testUndefineDefinedConnection(): void
     {
+        $this->expectException('InvalidArgumentException');
+        $this->expectExceptionMessage("Unknown connection scheme: 'tcp'");
+
         $factory = new Factory();
         $factory->undefine('tcp');
         $factory->create('tcp://127.0.0.1');
@@ -493,7 +510,7 @@ class FactoryTest extends PredisTestCase
     /**
      * @group disconnected
      */
-    public function testUndefineUndefinedConnection()
+    public function testUndefineUndefinedConnection(): void
     {
         $factory = new Factory();
         $factory->undefine('unknown');
@@ -504,12 +521,13 @@ class FactoryTest extends PredisTestCase
 
     /**
      * @group disconnected
-     * @expectedException \InvalidArgumentException
-     * @expecteExceptionMessage Unknown connection scheme: 'test'.
      */
-    public function testDefineAndUndefineConnection()
+    public function testDefineAndUndefineConnection(): void
     {
-        list(, $connectionClass) = $this->getMockConnectionClass();
+        $this->expectException('InvalidArgumentException');
+        $this->expectExceptionMessage("Unknown connection scheme: 'test'");
+
+        [, $connectionClass] = $this->getMockConnectionClass();
 
         $factory = new Factory();
 
@@ -520,61 +538,6 @@ class FactoryTest extends PredisTestCase
         $factory->create('test://127.0.0.1');
     }
 
-    /**
-     * @group disconnected
-     */
-    public function testAggregateConnectionSkipCreationOnConnectionInstance()
-    {
-        list(, $connectionClass) = $this->getMockConnectionClass();
-
-        $cluster = $this->getMock('Predis\Connection\Aggregate\ClusterInterface');
-        $cluster->expects($this->exactly(2))
-                ->method('add')
-                ->with($this->isInstanceOf('Predis\Connection\NodeConnectionInterface'));
-
-        $factory = $this->getMock('Predis\Connection\Factory', array('create'));
-        $factory->expects($this->never())
-                ->method('create');
-
-        $factory->aggregate($cluster, array(new $connectionClass(), new $connectionClass()));
-    }
-
-    /**
-     * @group disconnected
-     */
-    public function testAggregateConnectionWithMixedParameters()
-    {
-        list(, $connectionClass) = $this->getMockConnectionClass();
-
-        $cluster = $this->getMock('Predis\Connection\Aggregate\ClusterInterface');
-        $cluster->expects($this->exactly(4))
-                ->method('add')
-                ->with($this->isInstanceOf('Predis\Connection\NodeConnectionInterface'));
-
-        $factory = $this->getMock('Predis\Connection\Factory', array('create'));
-        $factory->expects($this->exactly(3))
-                ->method('create')
-                ->will($this->returnCallback(function ($_) use ($connectionClass) {
-                    return new $connectionClass();
-                }));
-
-        $factory->aggregate($cluster, array(null, 'tcp://127.0.0.1', array('scheme' => 'tcp'), new $connectionClass()));
-    }
-
-    /**
-     * @group disconnected
-     */
-    public function testAggregateConnectionWithEmptyListOfParameters()
-    {
-        $cluster = $this->getMock('Predis\Connection\Aggregate\ClusterInterface');
-        $cluster->expects($this->never())->method('add');
-
-        $factory = $this->getMock('Predis\Connection\Factory', array('create'));
-        $factory->expects($this->never())->method('create');
-
-        $factory->aggregate($cluster, array());
-    }
-
     // ******************************************************************** //
     // ---- HELPER METHODS ------------------------------------------------ //
     // ******************************************************************** //
@@ -582,19 +545,19 @@ class FactoryTest extends PredisTestCase
     /**
      * Returns a mocked Predis\Connection\NodeConnectionInterface.
      *
-     * @return array Mock instance and class name
+     * @return array Mock instance of a single node connection and its FQCN
      */
     protected function getMockConnectionClass()
     {
-        $connection = $this->getMock('Predis\Connection\NodeConnectionInterface');
+        $connection = $this->getMockBuilder('Predis\Connection\NodeConnectionInterface')->getMock();
 
-        return array($connection, get_class($connection));
+        return [$connection, get_class($connection)];
     }
 
     /**
      * Provides empty values for specific parameters.
      *
-     * These parameters usually trigger the addition of initializatin commands
+     * These parameters usually trigger the addition of initialization commands
      * to connection instances like `password` => AUTH and `database` => SELECT,
      * but they should not be added when their values are NULL or empty strings.
      *
@@ -602,16 +565,16 @@ class FactoryTest extends PredisTestCase
      */
     public function provideEmptyParametersForInitializationCommands()
     {
-        return array(
+        return [
             // AUTH
-            array('username', ''),
-            array('username', null),
-            array('password', ''),
-            array('password', null),
+            ['username', ''],
+            ['username', null],
+            ['password', ''],
+            ['password', null],
 
             // SELECT
-            array('database', ''),
-            array('database', null),
-        );
+            ['database', ''],
+            ['database', null],
+        ];
     }
 }
